/* -------------------- spi_uart.c -----------------

     stellt einfacheFunktionen fuer SPI und UART
     Datentransfer zur Verfuegung.

     15.04.2016   R. Seelig
  -------------------------------------------------- */

#include "spi_uart.h"

/* ----------------------------------------------------------
   SPI_INIT

     initialisiert die SPI Hardware des AVR Controllers

     speed => Taktgeschwindigkeit SPI-Bus (auch abhaengig
              von clkx2)

                    clkx2 = 0        clkx2 = 1

                0 : F_CPU / 4        F_CPU / 2
                1 : F_CPU / 16       F_CPU / 8
                2 : F_CPU / 64       F_CPU / 32
                3 : F_CPU / 128      F_CPU / 64
   ---------------------------------------------------------- */
void spi_init(uint8_t speed, uint8_t clkx2)
{
  SPI_DDR |= (1 << SPI_MOSI_PIN) | (1 << SPI_CLK_PIN) | (1 << SPI_SS_PIN) | (1 << SPI_RST_PIN);

  SPI_PORT |= (1 << SPI_MISO_PIN);              // Pop-Up Widerstand fuer MISO
  SPI_DDR &= ~(1 << SPI_MISO_PIN);


  // Enable SPI Port: Keine Interrupts, MSB first, Master Modus


  if (clkinv) SPCR= (1 << CPOL);
         else SPCR= 0;

  SPCR |= (1 << SPE) | (1 << MSTR);


  // SPI-Taktrate

  if (speed & 0x01) { SPCR |= (1 << SPR0); }
  if (speed & 0x02) { SPCR |= (1 << SPR1); }

  if (clkx2) { SPSR = (1 << SPI2X); }
        else { SPSR = 0; }
}

/* ----------------------------------------------------------
   SPI_OUT

      Byte ueber SPI senden / empfangen
      data ==> zu sendendes Datum
   ---------------------------------------------------------- */

unsigned char spi_out(unsigned char data)
{
  SPDR=data;                                // Datum senden
  while( ( SPSR & 0x80 ) != 0x80 );         // ... und warten bis es gesendet ist
  return SPDR;                              // empfangenes Byte ist Rueckgabewert
}


void spi_setrst(void)
{
  if (rinv)
    SPI_PORT &= ~(1 << SPI_RST_PIN);
  else
    SPI_PORT |= (1 << SPI_RST_PIN);
}

void spi_clrrst(void)
{
  if (!rinv)
    SPI_PORT &= ~(1 << SPI_RST_PIN);
  else
    SPI_PORT |= (1 << SPI_RST_PIN);
}


/* --------------------------------------------------
    serielle Schnittstelle initialisieren
   -------------------------------------------------- */


/* --------------------------------------------------
    Initialisierung der seriellen Schnittstelle:

    Uebergabe: baud = Baudrate
    Protokoll: 8 Daten-, 1 Stopbit
   -------------------------------------------------- */

void uart_init(uint32_t baud)
{
  uint16_t ubrr;

  if (baud> 57600)
  {
    baud= baud>>1;
    ubrr= (F_CPU/16/baud);
    ubrr--;
    UCSR0A |= 1<<U2X0;                                  // Baudrate verdoppeln
  }
  else
  {
    ubrr= (F_CPU/16/baud-1);
  }
  UBRR0H = (uint8_t)(ubrr>>8);                    // Baudrate setzen
  UBRR0L = (uint8_t)ubrr;

#if defined (__AVR_ATmega328P__) || defined (__AVR_ATmega168__) || defined (__AVR_ATtiny2313__)

  UCSR0B = (1<<RXEN0)|(1<<TXEN0);                       // Transmitter und Receiver enable
  UCSR0C = (3<<UCSZ00);                                 // 8 Datenbit, 1 Stopbit

#else

  // ATmega8
  UCSR0B |= (1<<RXEN0) | (1<<TXEN0);
  UCSRC  = (1<<URSEL) | (1<<UCSZ01) | (1<<UCSZ00);

#endif
}

/* --------------------------------------------------
    Zeichen ueber die serielle Schnittstelle senden
   -------------------------------------------------- */

void uart_putchar(uint8_t ch)
{
  while (!( UCSR0A & (1<<UDRE0)));                      // warten bis Transmitterpuffer leer ist
  UDR0 = ch;                                            // Zeichen senden
}

/* --------------------------------------------------
    testen, ob ein Zeichen auf der Schnittstelle
    ansteht
   -------------------------------------------------- */

uint8_t uart_ischar( void )
{
  return (UCSR0A & (1<<RXC0));
}

/* --------------------------------------------------
    Zeichen von serieller Schnittstelle lesen
   -------------------------------------------------- */

uint8_t uart_getchar( void )
{
  while(!(UCSR0A & (1<<RXC0)));                         // warten bis Zeichen eintrifft

#ifdef send_echo

  char ch;
  ch= UDR0;
  uart_putchar(ch);
  return ch;

#else

  return UDR0;

#endif
}


/* --------------------------------------------------
     RS232_PUTRAMSTRING
     gibt einen String aus dem RAM ueber die RS-232
     aus.
   -------------------------------------------------- */

void uart_putramstring(uint8_t *p)
{
  do
  {
    uart_putchar( *p );
  } while( *p++);
}

/* --------------------------------------------------
     RS232_PUTROMSTRING
     gibt einen String aus dem ROM ueber die RS-232
     aus.

     Bsp.:

          rs232_putromstring(PSTR("Hallo Welt")
   -------------------------------------------------- */

void uart_putromstring(const uint8_t *dataPtr)
{
  uint8_t c;

  for (c=pgm_read_byte(dataPtr); c; ++dataPtr, c=pgm_read_byte(dataPtr))
  {
    uart_putchar(c);
  }
}

/* --------------------------------------------------
     PRINTHEX
     gibt einen unsigned char als Hex-Wert auf der
     seriellen Schnittstelle aus
   -------------------------------------------------- */
void printhex(uint8_t value)
{
  uint8_t b;

  b= (value >> 4);
  if (b< 10) b += '0'; else b += 'A'-10;
  uart_putchar(b);
  b= value & 0x0f;
  if (b< 10) b += '0'; else b += 'A'-10;
  uart_putchar(b);
}

/* --------------------------------------------------
     GETHEX
     liest einen Hex-Wert von der seriellen Schnitt-
     stelle ein

     echo => 1 : gelesener Wert wird zurueckgesendet
             0 : nicht zuruecksenden
   -------------------------------------------------- */
uint8_t gethex(char echo)
{
  uint8_t ch;
  uint8_t value;

  ch= uart_getchar();
  if (echo) uart_putchar(ch);
  if (ch > 'F')
  {
    ch= (ch-'a')+10;
  }
  else
  {
    if (ch> '9') ch= (ch-'A')+10; else ch -= '0';
  }
  value= (ch<< 4);
  ch= uart_getchar();
  if (echo) uart_putchar(ch);

  if (ch > 'F')
  {
    ch= (ch-'a')+10;
  }
  else
  {
    if (ch> '9') ch= (ch-'A')+10; else ch -= '0';
  }
  value |= ch;
  return value;
}

void printhex16(uint16_t value)
{
  printhex(value >> 8);
  printhex(value & 0x00ff);
}

uint16_t gethex16(char echo)
{
  uint16_t i, i2;

  i= gethex(0);
  i= i & 0x00ff;
  i2= gethex(0);
  i2= i2 & 0x00ff;
  i2= i2 | (i << 8);
  if (echo) printhex16(i2);
  return i2;
}


/* --------------------------------------------------
     READSTR
     liest einen String von der seriellen Schnitt-
     stelle ein
   -------------------------------------------------- */
void readstr(char *string, char anz)
{
  char cnt, ch;

  cnt= 0;

  *string= 0;
  do
  {
    ch= uart_getchar();
    if (ch!= 0x0d)
    {
      if (ch!= 0x08)
      {
        if (cnt< anz)
        {
          uart_putchar(ch);
          *string= ch;
          string++;
          *string= 0;
          cnt++;
        }
      }
      else
      {
        if (cnt)
        {
          uart_putchar(0x08);
          string--;
          *string= 0;
          cnt--;
        }
      }
    }
  } while (ch != 0x0d);
}

/* --------------------------------------------------
     CRLF
     erzeugt einen Carriagereturn und linefeed auf
     der seriellen Schnittstelle
   -------------------------------------------------- */

void crlf(void)
{
  uart_putchar(0x0d);
  uart_putchar(0x0a);
}
